/*!
 <PRE>
 SOURCE FILE : aves.cpp
 DESCRIPTION.: Algoritmo que avalia expressoes matematicas simples
 AUTHOR......: Raul Joaquim Camara dos Santos
 LOCATION....: DIMAp/UFRN.
 STARTED ON.: Setembro/2007

 TO COMPILE..: g++ -Wall aves.cpp -o aves
 LAST UPDATE.: 12 de setembro de 2007.
 </pre>
*/
/* ========================================================================= */
/* INCLUDES   	 				                             */
/* ------------------------------------------------------------------------- */

#include <iomanip>
#include <iostream>
#include <fstream>

#include "inf2posf.h"
#include "resultado.h"

using namespace std;
/*                       FUNCOES                             */
char* trataErros(int codigo_de_erro);

void mostrar_erro(char* entrada);

//! Funcao principal que executa o programa com uma entrada dada pelo usuario
int main()
{
	//! variavel que recebe cada linha 
	char linha[DEFAULT_SIZE];
	//! recebe a expressao na forma posfixa
	char* decodificado;
	//! resultado da expressao
	int resultado;
	//! avalia erro
	int codigo_de_erro = 0;
	//! ponteiro para a exibicao da mensagem de erro
	char* error_msg;
	//arquivo com o log de erros salvos 
	ofstream arquivo("log_erros.txt");
	//testa se houve sucesso ao abrir o arquivo
	if(!arquivo) {
		cerr << "Erro no arquivo!" << endl;
		return 0;
	}

	while(cin.getline(linha, DEFAULT_SIZE)) {//recebimento de cada linha no arquivo ou dada pelo usuario
		if (strlen(linha) == 0) { //se a linha estiver vazia, passa
			continue;
		}

		decodificado = Posfixa(linha, DEFAULT_SIZE); //recebe o valor na forma posfixa

		//Houve erro na converso
		if (decodificado[0] == 'E') {
			
			mostrar_erro(decodificado);
			arquivo << decodificado << endl;
		}
		//nova avaliacao
		else {
			codigo_de_erro = 0;
			resultado = avaliaPosfixa(decodificado,
				&codigo_de_erro);

			delete [] decodificado;
			//Houve um erro, um tratamento e dado
			if (codigo_de_erro) {
				error_msg = trataErros(codigo_de_erro);

				cout<<linha<<" ";
				mostrar_erro(error_msg); //exibicao do erro
				arquivo << error_msg << endl; //salva no arquivo

			}
			else {
				//impressao e salvamento do resultado no arquivo.
				cout <<linha<<setw(30);
				cout<<" >> O resultado e': "<< resultado << endl;
				arquivo<< resultado << endl;
			}
		}
	}
	arquivo.close();
	return(EXIT_SUCCESS);
}

//! Funo para lidar com os erros matemticos da expresso
 /*! @param codigo_de_erro o codigo do erro
  */
char* trataErros(int codigo_de_erro)
{
	char* error_msg;
	switch (codigo_de_erro) {
		case ERRO_OPERANDO_DUPLO:
			error_msg = "E5 C";
			break;
		case ERRO_DIV_POR_ZERO:
			error_msg = "E8 C";
			break;
		case ERRO_CONSTANTE_INVALIDA:
			error_msg = "E1 C1";
			break;
		default:
			break;
	}

	return error_msg;
}

//! Funo para mostrar erros
 /*! @param entrada a mensagem
  */
void mostrar_erro(char* entrada)
{	
	cout << entrada;
	switch (*(entrada+1)){
		case '1':
			cout<<" - Constante numerica invalida"<<endl;
			break;
		case '2':
			cout<<" - Falta Operando"<<endl;
			break;
		case '3':
			cout<<" - Operando invalido"<<endl;
			break;
		case '4':
			cout<<" - Falta Operador"<<endl;
			break;
		case '5':
			cout<<" - Falta Operador"<<endl;
			break;
		case '6':
			cout<<" - Fechamento de escopo invalido"<<endl;
			break;
		case '7':
			cout<<" - Escopo aberto"<<endl;
			break;
		case '8':
			cout<<" - Divisao por zero"<<endl;
			break;

		default:
			break;
	}
}
