/*!
 <PRE>
 SOURCE FILE : inf2posf.h
 DESCRIPTION.: Algoritmo com funcoes para converter expressoes matematicas da forma infixa para a posfixa
 AUTHOR......: Raul Joaquim Camara dos Santos
 LOCATION....: DIMAp/UFRN.
 STARTED ON.: Setembro/2007

 TO COMPILE..: Nao compilavel.
 LAST UPDATE.: 11 de setembro de 2007.
 </pre>
*/
#include <iostream>
#include<stdio.h>
#include <new>
using std::bad_alloc;
#include <cassert>
using namespace std;

#define DEFAULT_SIZE 256

#include "stack.h"
// FUNCOES
char * Posfixa (const char * algebrica, const int size);
bool prcd( char op1, char op2);
int ImpOperador(char avalia);
char * Erro(char * NovaExpPosfixa, char * msg);

 //! Funo para avaliar a precedncia de um operador
 /*! @param avalia o operador
  *  @return o valor inteiro da "importancia" do operador
  */

int ImpOperador(char avalia)
{
	switch ( avalia )
	{
		case '(':   return 7;
		case ')':   return 7;
		case '^':  return 3;
		case '*':   return 2;
		case '/':    return 2;
		case '%':  return 2;
		case '+':  return 1;
		case '-':    return 1;
		case ' ':    return 5;
		default :  return 11;
	}
}

//! Funo para comparar a precedncia de dois operadores
 /*! @param ant um operador a ser avaliado
  *  @param prox outro operador a ser avaliado
  *  @return true se a precede b a false caso contrrio
  */
bool prcd( char op1, char op2)
{
	if ( ImpOperador(op1) >= ImpOperador(op2) ) 
		return true; 
		else return false;
}

//! Funo para destruir o vetor de retorno e mandar uma mensagem de erro
 /*! @param NovaExpPosfixa um vetor a ser destrudo
  *  @param msg uma mensagem de erro
  *  @return a prpria msg
  */

char * Erro(char * NovaExpPosfixa, char * msg)
{
	delete [] NovaExpPosfixa;	
	return msg;
}


//! Funcao de conversao da notacao infixa para notacao posfixa
 /*! @param algebrica string na notao infixa
  *  @param size o tamanho mximo da expresso
  *  @return a string na notao posfixa
  */
char * Posfixa (const char * algebrica, const int size)
{
	//! Pilha de operandos
	StackAr <char> OP(DEFAULT_SIZE);
	//esvazia a pilha
	OP.makeEmpty();	
	//! n  a varivel da leitura da string, count  a varivel de escrita em NovaExpPosfixa
	int n = 0, count = 0, i= 0;
	////!variavel para tratamento do '-' unario
	int next=0;
	 //! indica se est ou no dentro de parenteses e quantos so eles
	int abreprnts = 0, fechaprnts = 0;
	//! posicao do parntese inicial
	int prnts_begin = 0; 
	//! string auxiliar para a recursao
	char * aux;
	//! respectivamente - variavel que recebe cada elemento da expressao que esta sendo percorrida, o elemento do topo
	char symb = '\0', topsymb = ' ';
	//! verdade se o ultimo caractere era um numero
	bool eranumero = false; 
	//! string de retorno (nova lista na forma posfixa)
	char * NovaExpPosfixa; 
	
	//inicializacao da string
	try
	{
		NovaExpPosfixa = new char[DEFAULT_SIZE];
	}
	catch (const bad_alloc& exception)
	{
		cerr << "Expressao nao inicializada corretamente";
		assert(false);
	}

	while ( n < size)
	{		
		if (algebrica[n] == '\0' ) break;
			symb = algebrica[n];
		
		
		if ( isdigit(symb) ) //se symb for um operando
		{
			// Inclui symb na string posfixa
			if (abreprnts == fechaprnts)
			{
				NovaExpPosfixa[count] = symb;
				count++;				
			}
			eranumero = true;
		}
		//tratamento do '-' unario
		else if (next==1)
			{
				next=0;
				continue;
			}
		else if (symb =='+' && algebrica[n+1] == '-')
			{				
				symb ='-';
				next = 1;
			}
		else if (symb =='-' && algebrica[n+1] == '+')
			{				
				symb ='-';
				next = 1;
			}
		else if (symb =='-' && algebrica[n+1] == '-')
			{				
				symb ='+';
				next = 1;
			}
		  
		//tratamento dos parenteses
		else if (symb == '(')
			{
			if (abreprnts == fechaprnts)
				prnts_begin = n;
				abreprnts++;
			}
		else if (symb == ')')
			{
				fechaprnts++;
				if (abreprnts == fechaprnts) //ou seja, acabaram-se os ')'
				{
					abreprnts =0;
					fechaprnts=0;
					//recursao da substring dos parenteses
					aux = Posfixa( &algebrica[prnts_begin + 1], (n - prnts_begin - 1) );
					
					// Se a recursao detectar um erro, pra tudo e retorna a mensagem
					if (aux[0] == 'E')
					{
						delete [] NovaExpPosfixa;
						return aux;
					}
					for (i = 0; i < (int) strlen(aux); i++)
					{
						
						NovaExpPosfixa[count] = aux[i];
						count++;
					}
					delete [] aux;
				}
			}
							
		else //formacao da expressao posfixa
		{
			
			while ( (!(OP.isEmpty())) && prcd(OP.top(), symb) && (abreprnts == fechaprnts))
			{				
					topsymb = OP.pop();
					//incluir topsymb na string posfixa
					NovaExpPosfixa[count] = topsymb;
					count++;
			}
			

			if (abreprnts == fechaprnts)
			{
				
				
				
				if ( (eranumero == true) && (ImpOperador(symb) < 5) )
				{
					eranumero = false;
					if (abreprnts == fechaprnts)
					{
						//Organiza colocando um espao
						NovaExpPosfixa[count] = ' ';
						count++;
						
					}
				}
				else if (ImpOperador(symb) < 5)
					{
					if ( (n == 0) && (ImpOperador(symb)!=1) )
						{
						//Erro E2- Falta operando
						
						return Erro (NovaExpPosfixa ,"E2 C");
						}
					else if ( (ImpOperador(symb) == 1) && (n ==0) )
						{
							//adiciona 0 antes de + ou - unitrio
							NovaExpPosfixa[count] = '0';
							NovaExpPosfixa[count+1] = ' ';
							count+=2;
							
						}
					else if (n != 0)
						//Erro E2- Falta operando
						return Erro(NovaExpPosfixa, "E2 C");			
					}
				else if ((ImpOperador(symb) == 5) && eranumero)
					{
						//separa operandos por espao
						NovaExpPosfixa[count] = ' ';
						count++;
					}
				//Erro E3 - Operador Invalido
				if (!isdigit(symb) && (ImpOperador(symb) > 5) )
					return Erro(NovaExpPosfixa, "E3 C");
				OP.push(symb);
			}
		}
		
		n++;
	}	
	
	
	//Erro E2- Falta operando 	
	if ( (ImpOperador(symb) < 5) && eranumero)
		return Erro(NovaExpPosfixa, "E2 C");
	i = 0;							

	//Erro E7 - Escopo aberto
	if (abreprnts > 0) return Erro(NovaExpPosfixa, "E7 C");
	
	//Erro E6 - Fechamento de escopo invalido
	if (fechaprnts > 0) return Erro(NovaExpPosfixa, "E6 C");


	while (!OP.isEmpty()) 
	{
										
		topsymb = OP.pop();
		//incluir topsymb na string posfixa
		NovaExpPosfixa[count] = topsymb;
		count++;
	}
	NovaExpPosfixa[count] = '\0';
	
	return NovaExpPosfixa;
}
