/*!
 <PRE>
 SOURCE FILE : resultado.h
 DESCRIPTION.: Algoritmo que calcula o valor de uma expressao matematica na forma posfixa.
 AUTHOR......: Raul Joaquim C dos Santos.
 LOCATION....: DIMAp/UFRN.
 STARTED ON.: Setembro, 2007.

 TO COMPILE..: Nao compilavel.

 LAST UPDATE.: 11 de setembro de 2007.
 </pre>
*/
#include <cmath>
#include <ctype.h>

#include "stack.h"

#define DEFAULT_SIZE 256

#define ERRO_OPERANDO_DUPLO 1

#define ERRO_DIV_POR_ZERO 2

#define ERRO_CONSTANTE_INVALIDA 3

//! Funo para converter uma expresso posfixa em um resultado
 /*! @param a expressao a ser avaliada
  *  @param msg_erro Arquivo onde ficaram gravados as mensagens de erro
  *  @return o valor inteiro do resultado
  */

int avaliaPosfixa(char* entrada, int* msg_erro)
{

	//! Pilha de inteiros definida em stack.h
	StackAr<int> pilha(DEFAULT_SIZE);
	//! variavel para saber se um caractere e um numero
	bool insidenum = false;
	//! Posicao onde o numero comecou
	int begin_num = 0;
	//! Numero lido a ser inserido na pilha
	int num_lido;
	//! operandos a serem avaliados 
	int op1, op2;
	//! Caractere que esta sendo lido
	char pos_lida;
	//! valor retornado pela funcao
	int resultado;
	//! Usado para a impressao da msg de erro E1
	char *erro;
	erro = new char[256];

	//! loop principal. Leitura de cada caractere ate o fim
	for (int i = 0; ; i++) { 		// termina caso o caractere seja \0
		pos_lida = entrada[i];

		// se o caractere lido for um numero, se eu no estou dentro
		// de um numero, agora estou :)

		if (isdigit(pos_lida)) {
			if (!insidenum) {
				begin_num = i;
				insidenum = true;
			}
		}
		else {
			// acabo de sair de um numero

			if (insidenum) {
				num_lido = atoi(entrada+begin_num);
				
				insidenum = false;

				pilha.push(num_lido);
			}

			// verifica o caractere lido, procurando por operadores

			switch(pos_lida) {
				case '+':
					op2 = pilha.pop();
					op1 = pilha.pop();

					pilha.push(op1 + op2);

					break;
				case '-':
					op2 = pilha.pop();
					op1 = pilha.pop();

					pilha.push(op1 - op2);

					break;
				case '*':
					op2 = pilha.pop();
					op1 = pilha.pop();

					pilha.push(op1 * op2);

					break;
				case '/':
					op2 = pilha.pop();
					op1 = pilha.pop();

					// erro: diviso por zero: manda via pointer

					if (op2 == 0) {
						*msg_erro = ERRO_DIV_POR_ZERO;
						return 0;
					}

					pilha.push(op1 / op2);

					break;
				case '^':
					op2 = pilha.pop();
					op1 = pilha.pop();

					pilha.push((int) pow((float) op1, (float) op2));

					break;
				case '%':
					op2 = pilha.pop();
					op1 = pilha.pop();

					pilha.push(op1 % op2);

					break;
				default:
					break;
			}
		}
	
	if (entrada[i] == '\0')
		break;
	}

	// o resultado  o topo da pilha

	resultado = pilha.pop();

	// erro: se a pilha no for vazia (segundo caso), existem operandos duplos

	if (pilha.isEmpty())
		if (resultado < -32768 || resultado > 32767){
			*msg_erro = ERRO_CONSTANTE_INVALIDA;
			return 0; 
		}
		else return resultado;
	else {
		*msg_erro = ERRO_OPERANDO_DUPLO;
		return 0;
	}
}
