#ifndef STACKAR_H_
#define STACKAR_H_

#include "AbsStack.h"
#include "stack_exceptions.h"
#include <iostream>
using namespace std;
#include <new>
using std::bad_alloc;
#include <cassert>
/*!
 <PRE>
 SOURCE FILE : stack.h
 DESCRIPTION.: Classe que implementa uma pilha generica
 AUTHOR......: Raul Joaquim Camara dos Santos
 LOCATION....: DIMAp/UFRN.
 STARTED ON.: Setembro/2007
 CHANGES.....: None.

 LAST UPDATE.: 10 de setembro, 2007.
 </pre>
*/

/*! @class StackAr 
 *  @brief Classe que implementa uma pilha generica.
 *  Herda classe AbsStack.
*/
template<class Object>
class StackAr : public AbsStack <Object>
{
    private:
  int topo;
  Object * stack; 
  int Size;
  // Disable copy constructor
  StackAr( const StackAr &){ /* empty */}

  public:
//! Construtor padro
//! @param size O tamanho da pilha a ser alocada 

	StackAr( int size )
	{
		try
		{
			stack = new Object[size];
			Size = size;
			topo = -1;
		}
		catch (const bad_alloc& exception)
		{
			cerr << "Nao foi possivel incializar o vetor. Tente novamente mais tarde";
			assert(false);
		}
	}

//! Destrutor da pilha.
	~StackAr()
	{
		delete [] stack;
	}

//! Mtodo para insercao na pilha
/*! @param x o valor a ser inserido
 *  @exception OverflowException Se a pilha est cheia 
 */
	void push( const Object & x )
	{
		if ( (topo + 1) == Size ) throw OverflowException();
		topo++;
		stack[topo] = x;
	}


//! Mtodo para remocao do item do topo da pilha
/*! @return o ltimo valor inserido
 *  @exception UnderflowException caso pilha esteja vazia 
 */
	const Object & pop(void)
	{
		if ( topo == (-1)) throw UnderflowException();
        	topo--;
		return ( stack[topo + 1] );
	
	}

//! Funcao que retorna o topo da pilha
/*! @return o ltimo valor inserido
 *  @exception OverflowException caso a pilha esteja cheia 
 */
	const Object & top( ) const
	{
		if ( topo == (-1)) throw UnderflowException();
        	return stack[topo];	
	}
 
//! Funcao para saber se a pilha esta vazia
/*! @return Verdadeiro se e somente se a pilha est vazia.
*/
	bool isEmpty(void) const
	{
		if (topo <= -1) return true; 
		else return false;
	}

//! Esvazia a pilha
	void makeEmpty(void)
	{
		topo = -1;
	}


};

#endif
